#!/bin/sh
#
#= Fortran 90/95 dependency lister
#
AUTHORS="Yasuhiro MORIKAWA"
VERSION='$Id: f90depend.sh,v 1.1 2008/11/27 11:14:38 morikawa Exp $'
COPYRIGHT='Copyright (C) GFD Dennou Club, 2008. All rights reserved.'
#
# ȤƼ Fortran 90/95 ե use ʸ
# module ʸϤ, ¸ط򵭤ե Makefile 
# ɸϤ˽񤭽Ф.
#
# Ruby, Perl 󥹥ȡ뤵ƤʤĶǤ
# f90depend.rb, f90depend.pl إġ
# (θ Ruby ٤ƴŤ)
#

######################################################################
# Settings

# End Settings
######################################################################

f90dependhelp(){
    cat <<EOF

  Usage: 
    `basename $0` [OPTIONS] f90_file [f90_file [f90_file] .. ]

  Options
       -v:       verbose mode
       -q:       quiet mode
       -h:       show help messages

EOF

  f90dependprintversion
}

f90dependprintversion(){
    cat <<EOF
  `basename $0` Version ${VERSION}

  ${AUTHORS}
    ${COPYRIGHT}

EOF
}

#
# 
#
declare -a ArgvFiles
declare -a Files
declare -i Num=0
declare -i FNum=0
verbose=
while [ $# -gt 0 ]; do
    case $1 in
	"-v")
	    verbose=1
	    shift
	    ;;
	"-q")
	    verbose=
	    shift
	    ;;
	"-h")
	    f90dependhelp
	    exit 1
	    ;;
	"-"*)
	    echo "Error: \"$1\" is invalid option." >&2
	    echo "" >&2
	    exit 1
	    ;;
	*)
	    ArgvFiles[$Num]="$1"
	    Num="$Num + 1"
	    if [ ! -f "$1" ]; then
		echo "" >&2
		echo "  Error: \"$1\" is not found or not a regular file." >&2
		echo "" >&2
		exit 1
	    fi
	    shift
	    ;;
    esac
done

#
#  2 İʾʤϥإɽ
#
if [[ ${#ArgvFiles[*]} < 1 ]]; then
    f90dependhelp
    exit 1
fi

echo "  WARNING: this script will take a long time" >&2
echo "" >&2

#
# ʣե
#
Num=0
FNum=0
while [[ -n "${ArgvFiles[$Num]}" ]]; do
    new=1
    for f in ${Files[*]}; do
	if [ "$f" = "${ArgvFiles[$Num]}" ]; then
	    new=
	fi
    done
    if [ -n "$new" ]; then
	Files[$FNum]=${ArgvFiles[$Num]}
	FNum="$FNum + 1"
    fi
    Num="$Num + 1"
done

if [ -n "$verbose" ]; then
    echo "#  --- File list --- "
    Num=0
    while [[ -n "${Files[$Num]}" ]]; do
	echo "#  ${Files[$Num]}"
	Num="$Num + 1"
    done
    echo "#"
fi

#
#  2 İʾʤϥإɽ ()
#
if [[ ${#Files[*]} < 1 ]]; then
    f90dependhelp
    exit 1
fi


#
#  f90 ե뤫 USE ʸ
#
declare -a UseList
Num=0
while [[ -n "${Files[$Num]}" ]]; do
    UseList[$Num]=$( grep -iE "^[ ]*USE[ ]+" "${Files[$Num]}" | sed "s/^[ ]*USE[ ]\+//i" | cut -d ',' -f 1 | sort | uniq )
    Num="$Num + 1"
done

if [ -n "$verbose" ]; then
    echo "#  --- Use list --- "
    Num=0
    while [[ -n "${Files[$Num]}" ]]; do
	echo "#${Files[$Num]}: "
	echo "#   ${UseList[$Num]}" | tr -s '\n' ' '
	echo ""
	echo "#"
	Num="$Num + 1"
    done
    echo "#"
fi

#
#  f90 ե뤫 MODULE ʸ
#
declare -a ModuleList
Num=0
while [[ -n "${Files[$Num]}" ]]; do
    ModuleList[$Num]=$( grep -iE "^[ ]*MODULE[ ]+" "${Files[$Num]}" | grep -iv "^[ ]*MODULE[ ]\+PROCEDURE[ ]\+" | sed "s/^[ ]*MODULE[ ]\+//i" | cut -d ',' -f 1 | sort | uniq )
    Num="$Num + 1"
done

if [ -n "$verbose" ]; then
    echo "#  --- Module list --- "
    Num=0
    while [[ -n "${Files[$Num]}" ]]; do
	echo "#${Files[$Num]}: "
	echo "#   ${ModuleList[$Num]}" | tr -s '\n' ' '
	echo ""
	echo "#"
	Num="$Num + 1"
    done
    echo ""
fi

#
#  f90 ե뤫ᥤե븡
#
declare -a MainList
Num=0
while [[ -n "${Files[$Num]}" ]]; do
    MainList[$Num]=
    MainList[$Num]=$( grep -iE "^[ ]*PROGRAM[ ]+" "${Files[$Num]}" )
    if [ -n "${MainList[$Num]}" ]; then
	MainList[$Num]=$( echo "${Files[$Num]}" | sed 's/\.f9[05]$//i' )
    fi
    Num="$Num + 1"
done

if [ -n "$verbose" ]; then
    echo "#  --- Main list --- "
    Num=0
    while [[ -n "${Files[$Num]}" ]]; do
	echo "#${Files[$Num]}: "
	echo "#   ${MainList[$Num]}" | tr -s '\n' ' '
	echo ""
	echo "#"
	Num="$Num + 1"
    done
    echo ""
fi

#
# ֥ȥե̾κ
#
declare -a ObjFiles
Num=0
while [[ -n "${Files[$Num]}" ]]; do
    ObjFiles[$Num]=$( echo ${Files[$Num]} | sed 's/\.f9[05]$/.o/i' )
    Num="$Num + 1"
done


#
# USE ȥե̾Ѵ (¸ߤʤΤϺ)
#
declare -a DependList
Num=0
while [[ -n "${Files[$Num]}" ]]; do
    DependList[$Num]=
    for ul in ${UseList[$Num]}; do
	FNum=0
	while [[ -n "${Files[$FNum]}" ]]; do
	    for ml in ${ModuleList[$FNum]}; do
	        if [ "$ul" = "$ml" ]; then
		    DependList[$Num]="${DependList[$Num]} ${ObjFiles[$FNum]}"
	        fi
	    done
	    FNum="$FNum + 1"
	done
    done
    Num="$Num + 1"
done

if [ -n "$verbose" ]; then
    echo "#  --- Depend list --- "
    Num=0
    while [[ -n "${Files[$Num]}" ]]; do
	echo "#${Files[$Num]}: "
	echo "#   ${DependList[$Num]}" | tr -s '\n' ' '
	echo ""
	echo "#"
	Num="$Num + 1"
    done
    echo ""
fi

#
# ץ˴ؤ¸ꥹȺ
#
declare -a MainDepList
Num=0

while (( ${#MainList[*]} > $Num )) ; do
    if [ -z "${MainList[$Num]}" ]; then
	Num="$Num + 1"
	continue
    fi
    MainDepList[$Num]=
    for dl in ${DependList[$Num]}; do
	FNum=0
	while [[ -n "${ObjFiles[$FNum]}" ]]; do
	    if [ "$dl" = "${ObjFiles[$FNum]}" ]; then
		MainDepList[$Num]="${MainDepList[$Num]} ${ObjFiles[$FNum]} ${DependList[$FNum]}"
		FNum="$FNum + 1"
		continue
	    fi
	    FNum="$FNum + 1"
	done
    done
    MainDepList[$Num]=$( echo "${MainDepList[$Num]}" | tr -s ' ' '\n'| sort | uniq | tr -s '\n' ' ' )

    # ¿إå.
    PrevMainDepList=${MainDepList[$Num]}
    while true; do
	for dl in ${MainDepList[$Num]}; do
	    FNum=0
	    while [[ -n "${ObjFiles[$FNum]}" ]]; do
		if [ "$dl" = "${ObjFiles[$FNum]}" ]; then
		    MainDepList[$Num]="${MainDepList[$Num]} ${ObjFiles[$FNum]} ${DependList[$FNum]}"
		    FNum="$FNum + 1"
		    continue
		fi
		FNum="$FNum + 1"
	    done
	done
	MainDepList[$Num]=$( echo "${MainDepList[$Num]}" | tr -s ' ' '\n'| sort | uniq | tr -s '\n' ' ' )
	
	if [ "$PrevMainDepList" = "${MainDepList[$Num]}" ]; then
	    break
	else
	    PrevMainDepList=${MainDepList[$Num]}
	fi
    done
    
    Num="$Num + 1"
done

if [ -n "$verbose" ]; then
    echo "#  --- MainDep list --- "
    Num=0
    while [[ -n "${Files[$Num]}" ]]; do
	echo "#${Files[$Num]}: "
	echo "#   ${MainDepList[$Num]}" | tr -s '\n' ' '
	echo ""
	echo "#"
	Num="$Num + 1"
    done
    echo ""
fi

#
# 
#
Num=0
while [[ -n "${Files[$Num]}" ]]; do
    echo "${ObjFiles[$Num]}: ${DependList[$Num]}"
    Num="$Num + 1"
done
Num=0
while (( ${#MainList[*]} > $Num )) ; do
    if [ -z "${MainList[$Num]}" ]; then
	Num="$Num + 1"
	continue
    fi
    echo "${MainList[$Num]}: ${MainDepList[$Num]}"
    Num="$Num + 1"
done
